const DateHandler = require("../functions/DateHandler");
const Logger = require("./Logger");

class SystemLogger extends Logger {

  constructor() {
    super({
      fileName: 'system'
    });
    this.separator = ''.padEnd(10, '=');
    this.logDateFormat = 'DD/MM/YYYY HH:mm:ss';
    this.template = `${this.separator} [__SYSDATE__]\n__TYPE____ERROR__\n`;
    this.errorsPersistedInFiles = ['DB_ERROR'];
  }

  error(error) {
    if(!error) error = 'Ocorreu um erro inesperado. Parâmetro "error" não especificado';
    const err = typeof error === 'string' ? new Error(error) : error;
    if(this.errorsPersistedInFiles.includes(err.message)) return;
    console.error(err.message);
  }

  getLogByTemplate({ type, error }) {
    let logText = this.template;
    logText = logText.replace('__SYSDATE__', DateHandler.toString({ format: this.logDateFormat }));
    logText = logText.replace('__TYPE__', this.getHandledText({ label: 'TYPE', value: type }));
    logText = logText.replace('__ERROR__', this.getHandledText({ label: 'ERROR', value: error }));
    return logText.replace(/\n{2,}$/gi, '\n');
  }

  async write(error, info = {}, options = {}) {
    if(!error) error = 'Ocorreu um erro inesperado. Parâmetro "error" não especificado';
    const err = typeof error === 'string' ? new Error(error) : error;
    if(this.errorsPersistedInFiles.includes(err.message)) return;
    return super.write(error, info, options);
  }

  format(error, info) {
    const errorStr = this.getErrorMessage(error);
    return this.getLogByTemplate({...info, error: errorStr});
  }

  log(text) {
    console.log(text);
  }
}

module.exports = new SystemLogger();