const DB = require("../database/DB");

class UpdaterProdConfig {
  INSERT = `
    INSERT INTO
    UPDATERPRODCONFIG
    (
      CDPRODUCTID,
      NRVERSAO,
      CONFIG,
      DTINCLUSAO,
      DTULTATU
    )
    VALUES
    (
      :CDPRODUCTID,
      :NRVERSAO,
      :CONFIG,
      SYSDATE,
      SYSDATE
    )
  `;

  UPDATE = `
    UPDATE UPDATERPRODCONFIG
    SET
      CONFIG   = :CONFIG,
      DTULTATU = SYSDATE
    WHERE
      CDPRODUCTID = :CDPRODUCTID AND
      NRVERSAO    = :NRVERSAO
  `;

  DELETE_EMPTY_CONFIG_PRODUCT = `
    DELETE FROM
      UPDATERPRODCONFIG
    WHERE
      CDPRODUCTID = :CDPRODUCTID AND
      CONFIG IS NULL
  `;

  GET_LATEST_PROD_CONFIG = `
    SELECT
      NRVERSAO,
      CONFIG
    FROM
      UPDATERPRODCONFIG
    WHERE
      CDPRODUCTID = :CDPRODUCTID AND
      NRVERSAO <> NVL(:NRVERSAO, 'X.X.X')
    ORDER BY
      DTINCLUSAO DESC
  `;

  GET_PRODUCTS_WITH_CONFIG = `
    SELECT DISTINCT
      CDPRODUCTID
    FROM
      UPDATERPRODCONFIG
    WHERE
      CONFIG IS NOT NULL
  `;

  insert(params, executionParams = {}) {
    return DB.execute(this.INSERT, params, executionParams);
  }

  update(params, executionParams = {}) {
    return DB.execute(this.UPDATE, params, executionParams);
  }

  deleteEmptyConfigByProduct(params, executionParams = {}) {
    return DB.execute(this.DELETE_EMPTY_CONFIG_PRODUCT, params, executionParams);
  }

  getLatestProdConfig(params) {
    return DB.fetch(this.GET_LATEST_PROD_CONFIG, params);
  }

  getProductsWithConfig(params) {
    return DB.fetch(this.GET_PRODUCTS_WITH_CONFIG, params);
  }
}

module.exports = new UpdaterProdConfig();