const DB = require("../database/DB");

class UpdaterProdFili {
  INSERT = `
    INSERT INTO UPDATERPRODFILI
      ( UPDATERPRODID,  CDFILIAL, DTINCLUSAO, DTULTATU)
    VALUES
      (:UPDATERPRODID, :CDFILIAL, SYSDATE,    SYSDATE)
  `;

  UPDATE = `
    UPDATE
      UPDATERPRODFILI
    SET
      NRVERSAOATUAL = :P_NRVERSAO,
      DTULTATU      = SYSDATE
    WHERE
      ID = :P_ID
  `;

  GET_INSTALLED_PRODUCTS = `
    SELECT
      ID,
      UPDATERPRODID,
      NRVERSAOATUAL
    FROM
      UPDATERPRODFILI
    WHERE
      CDFILIAL = :CDFILIAL
  `;

  GET_INSTALLED_VERSIONS_BY_PRODUCT = `
    SELECT
      ID,
      CDFILIAL,
      NRVERSAOATUAL
    FROM UPDATERPRODFILI
    WHERE
      UPDATERPRODID = :UPDATERPRODID
  `;

  GET_VERSION = `
    SELECT
      ID
    FROM
      UPDATERPRODFILI
    WHERE
      CDFILIAL      = :P_CDFILIAL      AND
      UPDATERPRODID = :P_UPDATERPRODID AND
      NRVERSAOATUAL = :P_NRVERSAOATUAL
  `

  GET_MISSING_REQUIRED_PRODUCTS = `
    SELECT 
      UPF.CDFILIAL,
      UP.ID,
      UP.NMPRODUCT
    FROM 
      UPDATERPRODUTOS UP
      INNER JOIN UPDATERPRODFILI UPF ON ( UP.ID = UPF.UPDATERPRODID )
    WHERE 
      UPF.CDFILIAL = :CDFILIAL 
      AND UPF.UPDATERPRODID IN (SELECT TO_NUMBER(REGEXP_SUBSTR(:REQPRODS, '[^,]+', 1, LEVEL)) FROM DUAL CONNECT BY LEVEL <= LENGTH(:REQPRODS) - LENGTH(REPLACE(:REQPRODS, ',', '')) + 1)
      AND UPF.NRVERSAOATUAL IS NULL
  `;

  GET_UPDATER_PRODUCTS_SYS_MANAGEMENT = `
    SELECT
      PF.ID AS PRODFILIID,
      PF.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
      PF.CDFILIAL,
      F.NMFILIAL,
      E.SGESTADO,
      P.NMPRODUCT,
      P.IDUTMULTVER,
      P.ID,
      DECODE(PF.NRVERSAOATUAL, NULL, 'S', 'N') AS IDINSTPEND,
      CASE
        WHEN PF.NRVERSAOATUAL IS NULL AND EV.IDSTEVENTO IS NULL THEN
          'INSTALACAO_PENDENTE'
        WHEN EV.IDSTEVENTO IS NULL THEN
          'INSTALADO'    
        ELSE
          EV.IDSTEVENTO
        END AS IDSTEVENTO,
      DECODE(PF.NRVERSAOATUAL, NULL, '-', PF.NRVERSAOATUAL) AS NRVERSAOATUAL
    FROM
      UPDATERPRODUTOS P
    JOIN UPDATERPRODFILI PF
      ON (PF.UPDATERPRODID = P.ID)
    LEFT JOIN FILIAL F
      ON (PF.CDFILIAL = F.CDFILIAL)
    LEFT JOIN ENDEFILI E
      ON (
        E.CDFILIAL = F.CDFILIAL
        AND E.IDTPENDEFILI = 'P'
      )
    LEFT JOIN UPDATEREVENTOS EV
      ON (
        EV.CDPRODUCTID = P.ID
        AND EV.CDPRODFILIID = PF.ID
        AND EV.CDFILIAL = PF.CDFILIAL
        AND EV.IDSTEVENTO NOT IN ('FINALIZADO', 'CANCELADO')
      )
    WHERE
      P.ID = :ID
      AND (
        E.SGESTADO = :SGESTADO
        OR :SGESTADO IS NULL
      )
      AND (
        :STATUS IS NULL
        OR
        :STATUS = ( SELECT
            CASE
              WHEN (PF.NRVERSAOATUAL IS NULL AND EV.IDSTEVENTO IS NULL) THEN
                'INSTALACAO_PENDENTE'
              WHEN EV.IDSTEVENTO IS NULL THEN
                'INSTALADO'
            END
          FROM
            DUAL
        )
      )
    ORDER BY
      F.CDFILIAL ASC,
      PF.ID ASC
  `;

  insert(params) {
    return DB.execute(this.INSERT, params);
  }

  update(params, executionParams = {}) {
    return DB.execute(this.UPDATE, params, executionParams);
  }

  getInstalledProducts(params) {
    return DB.fetch(this.GET_INSTALLED_PRODUCTS, params);
  }

  getInstalledVersionsByProduct(params) {
    return DB.fetch(this.GET_INSTALLED_VERSIONS_BY_PRODUCT, params);
  }

  getVersion(params) {
    return DB.fetch(this.GET_VERSION, params);
  }
  
  getMissingRequiredProducts(params) {
    return DB.fetch(this.GET_MISSING_REQUIRED_PRODUCTS, params);
  }

  getUpdaterProductsSysManagement(params) {
    return DB.fetch(this.GET_UPDATER_PRODUCTS_SYS_MANAGEMENT, params);
  }
}

module.exports = new UpdaterProdFili();