const DB = require('../database/DB');

class UpdaterProdFiliCx {
  INSERT = `
    INSERT INTO UPDATERPRODFILICX
      ( UPDATERPRODID, CDFILIAL, CDCAIXA )
    VALUES
      ( :UPDATERPRODID, :CDFILIAL, :CDCAIXA )
  `;

  UPDATE = `
    UPDATE
      UPDATERPRODFILICX
    SET
      NRVERSAOATUAL = :P_NRVERSAO,
      DTULTATU      = SYSDATE
    WHERE
      ID = :P_ID
  `;

  GET_VERSION = `
    SELECT
      ID
    FROM
      UPDATERPRODFILICX
    WHERE
      CDFILIAL      = :P_CDFILIAL      AND
      CDCAIXA       = :P_CDCAIXA       AND
      UPDATERPRODID = :P_UPDATERPRODID AND
      NRVERSAOATUAL = :P_NRVERSAOATUAL
  `;

  GET_INSTALLED_PRODUCTS = `
    SELECT
      ID,
      UPDATERPRODID,
      NRVERSAOATUAL
    FROM
      UPDATERPRODFILICX
    WHERE
      CDFILIAL = :CDFILIAL AND
      CDCAIXA = :CDCAIXA
  `;

  GET_MISSING_REQUIRED_PRODUCTS = `
    SELECT
      UPFCX.CDFILIAL,
      UP.ID,
      UP.NMPRODUCT
    FROM
      UPDATERPRODUTOS UP
      INNER JOIN UPDATERPRODFILICX UPFCX ON ( UP.ID = UPFCX.UPDATERPRODID )
    WHERE
      UPFCX.CDFILIAL = :CDFILIAL
      AND UPFCX.CDCAIXA = :CDCAIXA
      AND UPFCX.UPDATERPRODID IN (SELECT TO_NUMBER(REGEXP_SUBSTR(:REQPRODS, '[^,]+', 1, LEVEL)) FROM DUAL CONNECT BY LEVEL <= LENGTH(:REQPRODS) - LENGTH(REPLACE(:REQPRODS, ',', '')) + 1)
      AND UPFCX.NRVERSAOATUAL IS NULL
  `;

  GET_UPDATER_PRODUCTS_SYS_MANAGEMENT = `
    SELECT
      UCX.ID AS PRODFILIID,
      F.CDFILIAL,
      F.NMFILIAL,
      F.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
      E.SGESTADO,
      C.CDCAIXA,
      C.NMCAIXA,
      C.CDCAIXA || ' - ' || C.NMCAIXA AS CDCAIXA_NMCAIXA,
      UP.ID AS CDPRODUCTID,
      UP.NMPRODUCT,
      UP.IDUTMULTVER,
      DECODE(UCX.NRVERSAOATUAL, NULL, 'S', 'N') AS IDINSTPEND,
      CASE
          WHEN UCX.NRVERSAOATUAL IS NULL AND EV.IDSTEVENTO IS NULL THEN
              'INSTALACAO_PENDENTE'
          WHEN EV.IDSTEVENTO IS NULL THEN
              'INSTALADO'
          ELSE
              EV.IDSTEVENTO
      END AS IDSTEVENTO,
      NVL(UCX.NRVERSAOATUAL, '-') AS NRVERSAOATUAL
    FROM
      UPDATERPRODFILICX UCX
      INNER JOIN FILIAL  F ON F.CDFILIAL = UCX.CDFILIAL
      LEFT JOIN ENDEFILI E ON E.CDFILIAL = F.CDFILIAL
                          AND E.IDTPENDEFILI = 'P'
      INNER JOIN CAIXA   C ON C.CDFILIAL = F.CDFILIAL
                          AND C.CDCAIXA = UCX.CDCAIXA
      INNER JOIN UPDATERPRODUTOS UP ON UCX.UPDATERPRODID = UP.ID
      LEFT JOIN UPDATEREVENTOSCX   EV ON EV.CDPRODUCTID    = UP.ID
                                  AND EV.CDFILIAL = F.CDFILIAL
                                  AND EV.CDCAIXA  = C.CDCAIXA
                                  AND EV.IDSTEVENTO NOT IN ( 'FINALIZADO', 'CANCELADO' )
    WHERE
      UCX.UPDATERPRODID = :ID
      AND UCX.CDFILIAL = :CDFILIAL
      AND ( :SGESTADO IS NULL OR E.SGESTADO = :SGESTADO )
      AND (
        :STATUS IS NULL
        OR
        :STATUS = (
          SELECT
            CASE
              WHEN (UCX.NRVERSAOATUAL IS NULL AND EV.IDSTEVENTO IS NULL) THEN
                'INSTALACAO_PENDENTE'
              WHEN EV.IDSTEVENTO IS NULL THEN
               'INSTALADO'
            END
          FROM DUAL
        )
      )

    ORDER BY
      UCX.CDCAIXA ASC,
      UCX.ID ASC
  `;

  insert(params, executionParams = {}) {
    return DB.execute(this.INSERT, params, executionParams);
  }

  update(params, executionParams = {}) {
    return DB.execute(this.UPDATE, params, executionParams);
  }

  getInstalledProducts(params) {
    return DB.fetch(this.GET_INSTALLED_PRODUCTS, params);
  }

  getVersion(params) {
    return DB.fetch(this.GET_VERSION, params);
  }

  getMissingRequiredProducts(params) {
    return DB.fetch(this.GET_MISSING_REQUIRED_PRODUCTS, params);
  }

  getUpdaterProductsSysManagement(params) {
    return DB.fetch(this.GET_UPDATER_PRODUCTS_SYS_MANAGEMENT, params);
  }
}

module.exports = new UpdaterProdFiliCx();