const DB = require("../database/DB");

class UpdaterServicoCx {

  getServiceMonitoringWithCasher(params) {
    const sql = `
      SELECT DISTINCT
        U.CDCAIXA, 
        C.CDCAIXA || ' - ' || C.NMCAIXA AS CAIXA
      FROM UPDATERSERVICOCX U 
        LEFT JOIN CAIXA C ON (C.CDCAIXA = U.CDCAIXA)      
      WHERE U.CDFILIAL = :CDFILIAL
  `;
    return DB.fetch(sql, params);
  }

  getServiceMonitoring(params) {
    const sql = `
      SELECT DISTINCT
        CDFILIAL,
        CDCAIXA,
        MONITORAMENTO
      FROM
        UPDATERSERVICOCX
      WHERE
        CDFILIAL = :P_CDFILIAL
        AND (
          CDCAIXA = :P_CDCAIXA
          OR :P_CDCAIXA = 'T' 
        )
        AND MONITORAMENTO IS NOT NULL
      ORDER BY CDCAIXA
    `;
    return DB.fetch(sql, params);
  }

  insert(params, executionParams = {}) {
    const sql = `
      INSERT INTO 
        UPDATERSERVICOCX (
          CDFILIAL, 
          CDCAIXA, 
          MONITORAMENTO, 
          NRORG, 
          DTINCLUSAO, 
          DTULTATU, 
          NRORGINCLUSAO, 
          NRORGULTATU
        ) 
      VALUES (
        :P_CDFILIAL,
        :P_CDCAIXA,
        :P_MONITORAMENTO,
        :P_NRORG,
        SYSDATE,
        SYSDATE,
        :P_NRORGINCLUSAO,
        :P_NRORGULTATU
      )
    `;
    return DB.execute(sql, params, executionParams);
  }

  update(params, executionParams = {}) {
    const sql = `
      UPDATE
          UPDATERSERVICOCX
      SET
        MONITORAMENTO = :P_MONITORAMENTO
      WHERE
        CDFILIAL = :P_CDFILIAL
        AND CDCAIXA  = :P_CDCAIXA
      `;
    return DB.execute(sql, params, executionParams);
  }

  getMonitoredUnits(params) {
    const sql = `
      SELECT
        F.CDFILIAL,
        F.NMFILIAL
      FROM
        UPDATERSERVICOCX U
        LEFT JOIN FILIAL F ON (F.CDFILIAL = U.CDFILIAL)
      WHERE
        MONITORAMENTO IS NOT NULL
      GROUP BY
        F.CDFILIAL,
        F.NMFILIAL
      ORDER BY
        F.CDFILIAL
    `;
    return DB.fetch(sql, params);
  }
}

module.exports = new UpdaterServicoCx();
