const DB = require('../database/DB');

class updaterProdVersaoModel 
{  
  getUpdaterProdVersao(updaterProdId) {
    const sql = `
      SELECT ID, NRVERSAO, DSURL, DSDIRETORIO
        FROM UPDATERPRODVERSAO
       WHERE UPDATERPRODID = :UPDATERPRODID
    `;

    const params = {
      UPDATERPRODID: updaterProdId
    };

    return DB.fetch(sql, params);
  }

  getUpdaterProdVersaoToDownload() {
    const sql = `
    SELECT 
      V.ID, 
      V.NRVERSAO, 
      V.DSURL, 
      V.DSDIRETORIO,
      V.IDDOWNLOADVER
    FROM 
      UPDATERPRODVERSAO V
      JOIN UPDATERPRODUTOS P ON P.ID = V.UPDATERPRODID
     WHERE 
      P.IDATIVO = 'S'
    `;

    return DB.fetch(sql);
  }

  getUpdaterProdVersaoByProduct(productId, useVersionFilter = process.env.USE_VERSION_FILTER) {
    const filter = useVersionFilter === 'true' ? " AND V.IDAPROVVER = 'S'" : "";

    const sql = `
      SELECT DISTINCT
        V.ID,
        V.NRVERSAO,
        V.DSURL,
        V.DSDIRETORIO,
        V.IDDOWNLOADVER,
        P.ID AS PRODUCT_ID
      FROM
        UPDATERPRODVERSAO V
        JOIN UPDATERPRODUTOS P ON P. ID = V.UPDATERPRODID
      WHERE
        P.ID = :ID
        ${filter}
      ORDER BY 
        V.NRVERSAO DESC
    `;

    const params = {
      ID: productId,
    }

    return DB.fetch(sql, params);
  }

  insertUpdaterProdVersao(params) {
    const sql = `
      INSERT INTO UPDATERPRODVERSAO
                ( UPDATERPRODID,  NRVERSAO,  DSURL,  DSDIRETORIO, DTINCLUSAO, DTULTATU)
         VALUES (:UPDATERPRODID, :NRVERSAO, :DSURL, :DSDIRETORIO, SYSDATE,    SYSDATE)
    `;
    return DB.execute(sql, params);
  }

  updateUpdaterProdVersaoStatus(params) {
    const sql = `
      UPDATE 
        UPDATERPRODVERSAO
      SET 
        IDDOWNLOADVER = :IDDOWNLOADVER
      WHERE 
        ID = :ID
    `;
    
    return DB.execute(sql, params);
  }

  getUpdaterProdVersaoConfig() {
    const sql = `
      SELECT 
        P.ID AS PRODUCT_ID, 
        P.NMPRODUCT, 
        V.ID, 
        V.NRVERSAO, 
        V.IDDOWNLOADVER, 
        V.IDAPROVVER
      FROM 
        UPDATERPRODVERSAO V
        JOIN UPDATERPRODUTOS P ON P.ID = V.UPDATERPRODID
      WHERE 
        P.IDATIVO = 'S'
    `;

    return DB.fetch(sql);
  }
  
  updateUpdaterProdVersaoAprov(params) {
    const sql = `
      UPDATE UPDATERPRODVERSAO
         SET IDAPROVVER = :IDAPROVVER
       WHERE ID = :ID
    `;
    return DB.execute(sql, params);
  }

 async getInstalledVersions(params) {
    const sql = `
      SELECT DISTINCT
          NRVERSAOATUAL AS NRVERSAO
      FROM
          UPDATERPRODFILI
      WHERE
          UPDATERPRODID = :UPDATERPRODID
          AND NRVERSAOATUAL IS NOT NULL
      ORDER BY
          NRVERSAOATUAL DESC
    `;

    return await DB.fetch(sql, params);
  }

  async getInstalledVersionsCasher(params) {
    const sql = `
      SELECT DISTINCT
        NRVERSAOATUAL AS NRVERSAO
      FROM
        UPDATERPRODFILICX
      WHERE
        UPDATERPRODID = :UPDATERPRODID
        AND NRVERSAOATUAL IS NOT NULL
      ORDER BY NRVERSAOATUAL
    `;

    return await DB.fetch(sql, params);
  }

  async getState() {
    const sql = `
      SELECT
        SGESTADO,
        SGESTADO || ' - ' || NMESTADO AS STATE
      FROM ESTADO
        WHERE CDPAIS = '0055'
      ORDER BY SGESTADO
    `;

    return await DB.fetch(sql);
  }
  
  async updateNote(params) {
    const sql = `    
      SELECT
        DSATUALIZACAO
      FROM UPDATERPRODVERSAO
        WHERE
          UPDATERPRODID = :UPDATERPRODID
          AND NRVERSAO = :NRVERSAO
    `;

    return await DB.fetch(sql, params);
  }
}

module.exports = new updaterProdVersaoModel();
