const SystemLogger = require("../logs/SystemLogger");
const jwt = require('../functions/JsonWebToken');
const AccessProfile = require('../service/AccessProfile');
const DbConnectionTest = require('../service/DbConnectionTest');

function APIRoute(router){
    router.post('/API/generateToken', (req, res) => {
        try {
            const token = jwt.sign(req.body);
            res.status(200).send({ token });
        } catch(error) {
            res.status(500).send({ error });
        }
    });
    router.get('/API/dbConnection', async (req, res) => {
        const { error, message } = await DbConnectionTest.executeTest();
        const statusCode = error ? 500 : 200;
        const response = error ? { error: message } : { message }; 
        res.status(statusCode).send(response);
    });
    router.post('/API/gracefulShutdown', () => {
        SystemLogger.log('Restarting app ...');
        process.exit(0);
    })
    router.post('/API/accessProfile', async (req, res) => {
        try {
            const accessProfile = await AccessProfile.getAccessProfile(req.body);
            res.status(200).send(accessProfile);
        } catch (error) {
            SystemLogger.write(error);
            res.status(500).send({ error });
        }
    });
}

module.exports = APIRoute;