const AssistantConfig = require('../model/AssistantConfig');
const Filial = require('../model/Filial');
const apiController = require('../controller/ApiController');
const updaterService = require('../service/UpdaterService');
const Gzip = require('../functions/Gzip');
const Query = require('../helpers/Query');
const ServicesMonitoring = require('../helpers/ServicesMonitoring');
const AssistantConfigController = require('../controller/AssistantConfigController');
const Updater = require('../socket/events/Updater');
const Monitor = require('../socket/events/Monitor');
const MonitoringCloud = require('../socket/events/pages/MonitoringCloud');
const MonitoringInterval = require('../socket/MonitoringInterval');

const route = (router) => {
  router.get('/data/:resource', async (req, res) => {
    try {
      const { resource } = req.params;
      const { page = 1, limit = 999, SGESTADO = '', search = '', search_in = '' } = req.query;
      switch( resource ) {
        case 'monitoring':
          const monitoringParams = await Filial.getUnits();
          const clientsConnected = await Monitor.getMonitors();
          const unitsConnected = clientsConnected.map(socket => socket.CDFILIAL );
          if(!monitoringParams) return;
          const { rows: data } = monitoringParams;
          data.forEach( item => {
            item.sat = item.SAT === 0 ? 'N' : 'E';
            item.websocket = unitsConnected.includes(item.CDFILIAL) ? 'S' : 'N';
          });
          return res.send({
            data,
            pagination: {
              limit: 10,
              total: data.length
            }
          });
        case 'orgProducts':
          const productsResult = await apiController.getOrgProducts();
          if(!productsResult) return;
          return res.send(productsResult);

        case 'parametrizacoes':
          let monitors = await AssistantConfigController.list();
          if (search && search_in) {
            const searchFields = Array.isArray(search_in) ? search_in : [search_in];
            monitors = monitors.filter((row) => {
              return searchFields.some(field =>
                (row[field] || "")
                .toString()
                .toLowerCase()
                .includes(search.toLowerCase())
              );
            });
          }

          res.send({
            data: monitors,
            pagination: {
              page,
              limit,
              total: monitors.length
            }
          });
          break;
        case 'parametrizacoesFilial':
          const units = await Filial.getActiveUnits();
          
          if(!units || !units.rows) {
            return;
          }

          const updaterSockets = await Updater.getUpdaters();
          const connectedUnits = [];

          updaterSockets.forEach((socket) => {
            if (!socket.CDCAIXA) {
              connectedUnits.push(socket.CDFILIAL);
            }
          });

          units.rows = await Promise.all(units.rows.map(async (unit) => {
            unit.CONNECTION_STATUS = connectedUnits.includes(unit.CDFILIAL);
            unit.WARNING_INFO = unit.WARNING ? UpdaterWarning.getActiveWarnings({
              unit: unit.CDFILIAL,
            }) : [];
            unit.WARNING = !!unit.WARNING_INFO.length;
  
            return unit;
          }));

          res.send({
            data: units.rows,
            pagination: {
              limit: 999,
              total: units.rows.length
            }
          });
          break
        case 'productsVersion':
          const productsVersion = await updaterService.getProductsVersion(req.query);
          if(productsVersion) {
            res.send({
              data: productsVersion.rows,
              pagination: {
                limit: 999,
                total: productsVersion.rows.length
              }
            });
          }
          break;
        case 'parametrizacoesApacheCloud' :
          const { rows } = await AssistantConfig.getMonitoringParamsByType({ TIPO: 'C' });
          const monitoringResult = MonitoringCloud.getMonitoringResults();
          rows.forEach(row => {
            let monitoring = monitoringResult[row.ID] || [];
            row.STATUS = monitoring.every(item => item.STATUS === 'S') ? 'S' : 'N';
          });
          res.send({
            data: rows,
            pagination: {
              limit: 999,
              total: rows.length
            }
          });
          break;
        case 'unitsWithCasher':
          const params = { SGESTADO };
          let { rows: unitsWithCasher } = await Filial.getUnitsWithCasher(params);
          if (search && Array.isArray(search_in)) {
            unitsWithCasher = unitsWithCasher.filter(row => {
              return search_in.some(key =>
                (row[key] || '')
                  .toString()
                  .toLowerCase()
                  .includes(search.toLowerCase())
              );
            });
          }
          res.send({
            data: unitsWithCasher,
            pagination: {
              limit: 999,
              total: unitsWithCasher.length
            }
          });
          break;
        default:
          res.status(404).send('ROUTE_NOT_FOUND');
      }
    } catch (error) {
      res.status(500).send({error: true, message: error.message});
    }
  });

  router.post('/data/:resource', async (req, res) => {
    const { resource } = req.params;
    try {
      switch (resource) {
        case 'monitoringConfig':

            const { COMANDO } = req.body;

            const COMANDO_ENCODED = Gzip.encode(COMANDO);

            if (COMANDO_ENCODED.length > 4000) {
              throw 'MAX_CHARACTER_REACHED_COMMAND';
            }
            const params = { ...req.body, COMANDO: COMANDO_ENCODED };
            delete params.ID;
            params.COMPLEMENTOTIPO = params.COMPLEMENTOTIPO || 'N';
            params.IDMONITORAMENTO = null;
            const result = await AssistantConfig.insert(params);

            const [RESULT_ID] = result.outBinds.ID;
            if (req.body.ATIVO === 'S') {
              const config = { ...req.body, ID: RESULT_ID };
              if(config.TIPO === 'C') {
                MonitoringCloud.updateMonitoringParams(config);
              } else {
                Monitor.updateMonitoringParams(config);
              }
            }
            res.send({
              error: false,
              params: {
                ID: RESULT_ID,
              },
              message: 'SUCCESS_INSERT_RECORD',
            });
          break;
        default:
          res.send(404).json({ error: true, message: 'RESOURCE_NOT_FOUND' });
      }
    } catch (error) {
      res.send({
        error: true,
        message: error.message,
      });
    }
  });

  router.put('/data/:resource/:ID', async (req, res) => {
    const { resource } = req.params;
    try {
      switch (resource) {
        case 'monitoringConfig':
          let { COMANDO, TIPO } = req.body;

          if(['L', 'A'].includes(TIPO) && !Query.validate(COMANDO)) {
            throw 'INVALID_COMMAND_QUERY';
          }
          if(TIPO === 'S') {
            COMANDO = ServicesMonitoring.removeMonitoringParamInconsistencies(COMANDO);
          }
          const COMANDO_ENCODED = Gzip.encode(COMANDO);

          if (COMANDO_ENCODED.length > 4000) {
            throw 'MAX_CHARACTER_REACHED_COMMAND';
          }

          await AssistantConfig.update({
            ...req.body,
            COMANDO: COMANDO_ENCODED,
          });

          const config = { ...req.body };
          if(config.TIPO === 'C') {
            MonitoringCloud.updateMonitoringParams(config);
          } else {
            Monitor.updateMonitoringParams(config);
          }
          res.send({
            error: false,
            message: 'SUCCESS_UPDATE_RECORD',
          });
          break;
        default:
          res.send(404).json({ error: true, message: 'RESOURCE_NOT_FOUND' });
      }
    } catch (error) {
      res.send({
        error: true,
        message: error.message,
      });
    }
  });

  router.delete('/data/:resource/:ID', async (req, res) => {
    const { resource, ID } = req.params;
    try {
      switch (resource) {
        case 'monitoringConfig':
          const idCollection = ID.split(',');
          const idToRemove = `_${idCollection.join('_')}_`;
          await AssistantConfig.delete({ ID: idToRemove });
          idCollection.forEach(removedId => {
            MonitoringInterval.removeById(removedId);
            MonitoringCloud.updateMonitoringParams({ ID: removedId });
          });
          res.send({
            error: false,
            message: 'SUCCESS_DELETE_RECORD',
          });
          break;
        default:
          res.status(404).send({ error: true, message: 'RESOURCE_NOT_FOUND' });
      }
    } catch (error) {
      res.status(500).send({
        error: true,
        message: error.message,
      });
    }
  });
}

module.exports = route;