const SystemLogger = require('../logs/SystemLogger');
const updaterModel = require('../model/updaterModel');
const Updater = require('../socket/events/Updater');

const route = (router) => {
  router.get('/installer/check-server', async (req, res) => {
    try {
        res.status(200).send({
            STATUS: true,
            VERSION: '0.0.0',
        });
    } catch (error) {
        SystemLogger.write(error);
        res.status(500).send({ error });
    }
  });
  router.post('/installer/validate', async (req, res) => {
    try {
      const { CDFILIAL, CDCAIXA = null } = req.body;
      const { socket, socketInfo } = await Updater.getUpdaterConnected(CDFILIAL, CDCAIXA);
      const { ip, hostname } = socketInfo || {};
      if (socket) {
        const deviceInfo = '' + ( ip && `IP: ${ip} `) + (hostname && `Hostname: ${hostname}`);
        if (CDCAIXA) {
          throw new Error(`O caixa selecionado já se encontra conectado ao Assistant em outro dispositivo. ${deviceInfo}`);
        }
        throw new Error(`A filial selecionada já se encontra conectada ao Assistant em outro dispositivo. ${deviceInfo}`);
      }
      const DEFAULT_USER = await updaterModel.getDefaultOperator({ P_CDFILIAL: CDFILIAL });
      const [ ROW ] = DEFAULT_USER.rows || [];

      if (DEFAULT_USER.rows.length === 0 || (!ROW.CDOPERADOR && !ROW.CDVENDEDOR)) {
        throw new Error(`Não foi possível encontrar operador/vendedor padrão para a filial selecionada`);
      }
      res.status(200).send({
        STATUS: true,
      });
    } catch (error) {
        SystemLogger.write(error);
        res.status(500).send({ ERROR_MSG: error.message });
    }
  });
}

module.exports = route;