class MonitoringInterval {
  static intervals = {};

  getUniqueKey({ unit, casher }) {
    const key = casher ? `${unit}_${casher}` : unit;
    return key;
  }

  add(id, callbackFn, intervalTimeMs, { unit, casher = null }) {
    const key = this.getUniqueKey({ unit, casher });
    if (!this.hasActive(id, { unit, casher })) {
      MonitoringInterval.intervals[key] = MonitoringInterval.intervals[key] || {};
      MonitoringInterval.intervals[key][id] = setInterval(callbackFn, intervalTimeMs);
    }
  }

  remove(id, { unit, casher = null }) {
    const key = this.getUniqueKey({ unit, casher });
    if (this.hasActive(id, { unit, casher })) {
      clearInterval(MonitoringInterval.intervals[key][id]);
      delete MonitoringInterval.intervals[key][id];
    }
  }

  removeById(id) {
    Object.keys(MonitoringInterval.intervals).forEach((uniqueKey) => {
      if (MonitoringInterval.intervals[uniqueKey] && MonitoringInterval.intervals[uniqueKey][id]) {
        clearInterval(MonitoringInterval.intervals[uniqueKey][id]);
        delete MonitoringInterval.intervals[uniqueKey][id];
      }
    })
  }

  clear() {
    Object.keys(MonitoringInterval.intervals).forEach((uniqueKey) => {
      Object.keys(MonitoringInterval.intervals[uniqueKey] || {}).forEach((id) => {
        const intervalId = MonitoringInterval.intervals[uniqueKey][id];
        clearInterval(intervalId);
      });
      delete MonitoringInterval.intervals[uniqueKey];
    });
  }

  hasActive(id, { unit, casher = null}) {
    const key = this.getUniqueKey({ unit, casher });
    if (MonitoringInterval.intervals[key]) {
      if (id && MonitoringInterval.intervals[key][id]) {
        return true;
      }
      if (!id && Object.keys(MonitoringInterval.intervals[key]).length !== 0) {
        return true;
      }
    }
    return false;
  }

  hasSomeActive() {
    return Object.keys(MonitoringInterval.intervals).some((uniqueKey) => {
      return Object.keys(MonitoringInterval.intervals[uniqueKey]).length !== 0;
    });
  }
}

module.exports = new MonitoringInterval();