const SocketRooms = require("../../SocketRooms");
const UpdaterService = require('../../../service/UpdaterService');
const Updater = require('../Updater');
const SystemLogger = require("../../../logs/SystemLogger");

class Parametrization {
  register(socket, { page }) {
    const { master } = SocketRooms;
    const roomName =
      page === 'parametrizations'
        ? master.parametrizationsPage
        : master.parametrizationsPerCasherPage;
    socket.join(roomName);

    socket.on('updateConfigIni', async (params) => {
      const { CDFILIAL: unit, CDCAIXA: casher, CONFIG } = params;
      Updater.emit('updateConfig', CONFIG, { unit, casher }).catch(e => SystemLogger.error(e));
    });

    socket.on('testConnection', async ({ CDFILIAL, CDCAIXA, PARAMS = {} }) => {
      try {
        let config = {};

        if (Object.keys(PARAMS).length === 0) {
          const params = {
            CDFILIAL,
            CDCAIXA: CDCAIXA || null,
          };

          config = await UpdaterService.getConfigIni(params);
        } else {
          config.data = {};
          config.data.database = PARAMS;
        }

        await Updater.emit('updaterTestConnection', config.data, { unit: CDFILIAL, casher: CDCAIXA }).catch(() => {
          throw new Error('Falha no teste de conexão. Não existe conexão ativa com a loja no momento.');
        });
      } catch (error) {
        socket.emit('testConnectionFinish', {
          error: true,
          message: error.message,
        });
      }
    });
  }
}

module.exports = new Parametrization();