const SocketRooms = require('../../SocketRooms');
const Updater = require('../Updater');
const UpdaterWarning = require('../../../helpers/UpdaterWarning');
const PortalEmitter = require('../PortalEmitter');
const SystemLogger = require('../../../logs/SystemLogger');
const ServiceInterval = require('../../ServiceInterval');
const SocketManager = require('../../SocketManager');
class ServiceManagement {
  register(socket, { page }) {
    const { master } = SocketRooms;
    const roomName =
      page === 'serviceManagement'
        ? master.serviceManagement
        : master.serviceManagementPerCasher;
    socket.join([master.default, roomName]);

    socket.on('detailStatusConnection', async (data) => {
      const { CDFILIAL: unit, CDCAIXA: casher = null } = data;
      const STATE = UpdaterWarning.getState({ unit, casher }) || {};
      const { DUPLICATE = {} } = STATE.DATA || {};
      socket.emit('detailStatusConnectionResult', DUPLICATE);
    });

    socket.on('removeWarning', async (data) => {
      const { CDFILIAL: unit, CDCAIXA: casher, WARNING_INFO } = data;
      UpdaterWarning.removeWarning({ unit, casher }, WARNING_INFO);
      const socketUpdater = await Updater.getUpdaterSocketByUnitAndCasher(unit, casher);
      const warnings = UpdaterWarning.getActiveWarnings({ unit, casher });

      const connected = !!socketUpdater;
      const socketData = {
        CDFILIAL: unit,
        CDCAIXA: casher,
        CONNECTED: connected,
        WARNING: !!warnings.length,
        WARNING_INFO: warnings,
      }
      PortalEmitter.emit('updaterConnected', socketData);
    });

    socket.on('execServiceManagement', async (params) => {
      try {
        const { CDFILIAL: unit, CDCAIXA: casher = null } = params;
        await Updater.emit('serviceManagement', params, { unit, casher });
      } catch (error) {
        SystemLogger.error(error);
      }
    });

    socket.on('execServiceStatusDetails', async (params) => {
      try {
        const { CDFILIAL, SERVICES, CDCAIXA = null, FILTERALL = false } = params;

        const oldRoom = [...socket.rooms].find(roomName => {
          return roomName.includes('ASSISTANT_MASTER_SERVICE_MANAGEMENT_');
        });

        if (oldRoom) {
          const splittedName = oldRoom.split('_');
          let casherByRoom = splittedName.pop();
          casherByRoom = casherByRoom === 'ALL' ? '' : casherByRoom;
          const unitByRoom = splittedName.pop();

          this.clearInterval(oldRoom, socket, unitByRoom, casherByRoom);
        }

        const selectedCasher = FILTERALL ? 'ALL' : CDCAIXA;
        socket.join(SocketRooms.master.statusServicesManagement({ CDFILIAL, CDCAIXA: selectedCasher }));
        if (FILTERALL) {
          Updater.emitAll('execServiceStatusDetails', SERVICES, { unit: CDFILIAL, casher: CDCAIXA })
                 .catch(e => SystemLogger.error(e));
        } else {
          Updater.emit('execServiceStatusDetails', SERVICES, { unit: CDFILIAL, casher: CDCAIXA })
                 .catch(e => SystemLogger.error(e));
        }

        ServiceInterval.createInterval('execServiceStatusDetails', { CDFILIAL, SERVICES, CDCAIXA, FILTERALL });
      } catch (error) {
        SystemLogger.error(error);
      }
    });

    socket.on('clearIntervalStatus', async ({ CDFILIAL, CDCAIXA = null }) => {
      const roomName = SocketRooms.master.statusServicesManagement({ CDFILIAL, CDCAIXA });
      this.clearInterval(roomName, socket, CDFILIAL, CDCAIXA);
    });

  }

  async clearInterval(roomName, socket, CDFILIAL, CDCAIXA) {
    socket.leave(roomName);
    const connectedUsers = await SocketManager.getRoom(roomName).fetchSockets();
    if (!connectedUsers.length) ServiceInterval.clearInterval(CDFILIAL, CDCAIXA);
  }
}

module.exports = new ServiceManagement();